package lightcontainer.domains.client;

import lightcontainer.interfaces.ClientHandlerFFE;

import java.io.*;
import java.net.Socket;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * ClientHandler
 *
 * <!> UNICAST CLIENT <!>
 * Class communicating with the client, and
 * intercepting and sending files to the client.
 *
 * @version     1.1
 * @since       1.0
 *
 * @see         Runnable
 * @see         AutoCloseable
 * @author      Jérémi NIHART <j.nihart@student.helmo.be>
 */
public class ClientHandler implements Runnable, AutoCloseable {
    // Variables
    private ClientHandlerFFE fileFrontEnd;
    private final Socket client;
    private boolean client_run;

    private BufferedReader reader;
    private PrintWriter writer;

    // Constructor
    public ClientHandler(Socket client, ClientHandlerFFE ffe) {
        this.fileFrontEnd = ffe;
        this.client = client;
        this.client_run = false;
        initClient();
    }

    /**
     * Initialise the Client's Reader and Writer.
     *
     * @since       1.0
     *
     * @see         BufferedReader
     * @see         PrintWriter
     */
    private void initClient() {
        try {
            this.reader = new BufferedReader(new InputStreamReader(
                    this.client.getInputStream(),
                    StandardCharsets.UTF_8
            ));
            this.writer = new PrintWriter(new OutputStreamWriter(
                    this.client.getOutputStream(),
                    StandardCharsets.UTF_8
            ), true);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Thread Function
     * Start the dialogue with the client.
     *
     * @since       1.0
     */
    @Override
    public void run() {
        this.client_run = true;
        while (this.client_run) {
            try {
                String command = this.reader.readLine();
                // TODO gestion de la réception de commandes, fichier, ...
                if (command != null) System.out.println("Client: " + command);
            } catch (IOException ignore) { }
        }
    }

    /**
     * AutoClosable Function
     * Close the Client thread and resources.
     *
     * @since       1.0
     */
    @Override
    public void close() {
        if (this.client_run) {
            try {
                this.client_run = false;
                this.client.close();
            } catch (IOException ignored) { }
        }
    }
}
