package lightcontainer.protocol;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class ProtocolReader {
    // Variables
    private final Pattern rulePattern;

    // Constructor
    protected ProtocolReader(String pattern) {
        this.rulePattern = Pattern.compile(pattern);
    }

    public abstract class ProtocolResult {}

    /**
     * Permet de lancer la décomposition d'une commande pour en extraire les données
     * @param data Contenu de la commande
     */
    public <T extends ProtocolResult> T execute(String data) {
        Matcher ruleMatcher = this.rulePattern.matcher(data);

        if (ruleMatcher.matches()) {
            String[] groups = new String[ruleMatcher.groupCount()];

            for (int i = 1; i <= groups.length; ++i)
                groups[i - 1] = ruleMatcher.group(i);

            return onExecuted(groups);
        }
        return null;
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     * @param data Paramètres pour créer la commande.
     */
    protected abstract <T> T onExecuted(String... data);
}
